import { isEqual, has, isString } from 'lodash'
import {
    REQUEST_START,
    REQUEST_KO, 
    SHOW_SPINNER,
    HIDE_SPINNER,
    SHOW_NOTIFICATION, 
    HIDE_NOTIFICATION, 
    REQUEST_OK
} from '../constants/ActionTypes';

const initialState = {
    spinner: {
        isOpen: false,
        message: null,
    },
    notification: {
        isOpen: false,
        message: null
    },
    dialog: {
        isOpen: false,
        message: null
    }
}

/**
 * 
 * @param {*} state 
 * @param {*} action 
 */
export const app = (state = initialState, action) => {
    if (isEqual(action.type, SHOW_SPINNER) || isEqual(action.type, REQUEST_START)) {
        state = {
            ...state,
            spinner: {
                isOpen: true,
                message: action.payload
            }
        }
    }

    if (isEqual(action.type, HIDE_SPINNER) || isEqual(action.type, REQUEST_OK)) {
        if (has(action.payload, 'action')) {
            if (isEqual(action.payload.action.type, 'redirect')) {
                const params = {
                    ip: action.payload.action.ip,
                    port: action.payload.action.port
                }

                window.open(`${window.location.protocol}//${params.ip}:${params.port}`, '_blank')
            }
        }   
        
        state = {
            ...state,
            spinner: {
                isOpen: false,
                message: isString(action.payload) ? action.payload : null
            }
        }
    }

    if (isEqual(action.type, SHOW_NOTIFICATION) || isEqual(action.type, REQUEST_KO)) {
        state = {
            ...state,
            spinner: {
                isOpen: false,
                message: null
            },
            notification: {
                isOpen: true,
                message: isString(action.payload) ? action.payload : 'No se pudo terminar la petición'
            }
        }
    }

    if (isEqual(action.type, HIDE_NOTIFICATION)) {
        state = {
            ...state,
            notification: {
                isOpen: false,
                message: null
            }
        }
    }

    return state
}
